require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_sfauthenticationservice',
  dscmeta_resource_friendly_name: 'SFAuthenticationService',
  dscmeta_resource_name: 'VE_SFAuthenticationService',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'Storefront',
  dscmeta_module_version: '0.9.6',
  docs: 'The DSC SFAuthenticationService resource type.
         Automatically generated from version 0.9.6',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_virtualpath: {
      type: 'String',
      desc: 'Citrix Storefront authentication service IIS Virtual Path',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_siteid: {
      type: 'Optional[Integer[0, 18446744073709551615]]',
      desc: 'Citrix Storefront Site Id',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt64',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_friendlyname: {
      type: 'Optional[String]',
      desc: 'Authentication service friendly name',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
