require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_sfgateway',
  dscmeta_resource_friendly_name: 'SFGateway',
  dscmeta_resource_name: 'VE_SFGateway',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'Storefront',
  dscmeta_module_version: '0.9.6',
  docs: 'The DSC SFGateway resource type.
         Automatically generated from version 0.9.6',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_url: {
      type: 'String',
      desc: 'NetScaler gateway Url',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_stasbypassduration: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Time before retrying a failed STA server (seconds)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_subnetipaddress: {
      type: 'Optional[String]',
      desc: 'IP address',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'NetScaler gateway display name',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_secureticketauthorityurls: {
      type: 'Optional[Array[String]]',
      desc: 'Secure Ticket Authority server Urls',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_smartcardfallbacklogontype: {
      type: "Optional[Enum['UsedForHDXOnly', 'Domain', 'RSA', 'DomainAndRSA', 'SMS', 'GatewayKnows', 'SmartCard', 'None']]",
      desc: 'Login type to use when SmartCard fails',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_callbackurl: {
      type: 'Optional[String]',
      desc: 'NetScaler gateway authentication NetScaler call-back Url',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sessionreliability: {
      type: 'Optional[Boolean]',
      desc: 'Enable session reliability',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_requesttickettwostas: {
      type: 'Optional[Boolean]',
      desc: 'Request STA tickets from two STA servers (Requires two STA servers)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_logontype: {
      type: "Enum['UsedForHDXOnly', 'Domain', 'RSA', 'DomainAndRSA', 'SMS', 'GatewayKnows', 'SmartCard', 'None']",
      desc: 'Login type required and supported by the Gateway',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_stasuseloadbalancing: {
      type: 'Optional[Boolean]',
      desc: 'Load balance between the configured STA servers (requires two or more STA servers)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
