require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_sfstorefarm',
  dscmeta_resource_friendly_name: 'SFStoreFarm',
  dscmeta_resource_name: 'VE_SFStoreFarm',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'Storefront',
  dscmeta_module_version: '0.9.6',
  docs: 'The DSC SFStoreFarm resource type.
         Automatically generated from version 0.9.6',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_transporttype: {
      type: "Optional[Enum['HTTP', 'HTTPS', 'SSL']]",
      desc: 'Xml service transport type',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allfailedbypassduration: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Period of time to skip all xml service requests should all servers fail to respond',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_servers: {
      type: 'Array[String]',
      desc: 'Citrix XenDesktop delivery controllers',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_farmtype: {
      type: "Enum['XenApp', 'XenDesktop', 'AppController', 'VDIinaBox']",
      desc: 'The hostnames or IP addresses of the xml services',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_farmname: {
      type: 'String',
      desc: 'Farm display name',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_radetickettimetolive: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Period of time a RADE launch ticket is valid once requested on pre 7.0 XenApp and XenDesktop farms',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_storevirtualpath: {
      type: 'String',
      desc: 'Storefront Store virtual path',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_bypassduration: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Period of time to skip a server when is fails to respond',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_loadbalance: {
      type: 'Optional[Boolean]',
      desc: 'Round robin load balance the xml service servers',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_sslrelayserviceport: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Period of time to skip a server when is fails to respond',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_maxfailedserversperrequest: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Maximum number of servers within a single farm that can fail before aborting a request',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_port: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Xml service communication port',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_tickettimetolive: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Period of time an ICA launch ticket is valid once requested on pre 7.0 XenApp and XenDesktop farms',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
