require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_sfstoreregistergateway',
  dscmeta_resource_friendly_name: 'SFStoreRegisterGateway',
  dscmeta_resource_name: 'VE_SFStoreRegisterGateway',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'Storefront',
  dscmeta_module_version: '0.9.6',
  docs: 'The DSC SFStoreRegisterGateway resource type.
         Automatically generated from version 0.9.6',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_storevirtualpath: {
      type: 'String',
      desc: 'Citrix Storefront Authentication Service IIS Virtual Path',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_siteid: {
      type: 'Optional[Integer[0, 65535]]',
      desc: 'Citrix Storefront Authentication Service IIS Site Id',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt16',
      mof_is_embedded: false,
    },
    dsc_gateway: {
      type: 'String',
      desc: 'Citrix Storefront Netscaler Gateway',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_usefullvpn: {
      type: 'Optional[Boolean]',
      desc: 'Use full VPN access when accessing the Citrix Storefront Store through this Gateway',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_defaultgateway: {
      type: 'Optional[Boolean]',
      desc: 'Use this Gateway as the default if more than one is associated to the Citrix Storefront Store',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
